/**
 * Copyright (c) 2014 - 2019, Nordic Semiconductor ASA
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form, except as embedded into a Nordic
 *    Semiconductor ASA integrated circuit in a product or a software update for
 *    such product, must reproduce the above copyright notice, this list of
 *    conditions and the following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. Neither the name of Nordic Semiconductor ASA nor the names of its
 *    contributors may be used to endorse or promote products derived from this
 *    software without specific prior written permission.
 *
 * 4. This software, with or without modification, must only be used with a
 *    Nordic Semiconductor ASA integrated circuit.
 *
 * 5. Any software provided in binary form under this license must not be reverse
 *    engineered, decompiled, modified and/or disassembled.
 *
 * THIS SOFTWARE IS PROVIDED BY NORDIC SEMICONDUCTOR ASA "AS IS" AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL NORDIC SEMICONDUCTOR ASA OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */
 
// Header "include guards"
#ifndef ISP1807_H_INCLUDED
#define ISP1807_H_INCLUDED

#include <stdint.h>
#include <string.h>
#include "nordic_common.h"
#include "nrf.h"
#include "ble_hci.h"
#include "ble_advdata.h"
#include "ble_advertising.h"
#include "ble_conn_params.h"
#include "nrf_sdh.h"
#include "nrf_sdh_soc.h"
#include "nrf_sdh_ble.h"
#include "nrf_ble_gatt.h"
#include "nrf_ble_qwr.h"
#include "app_timer.h"
#include "ble_nus.h"
#include "app_uart.h"
#include "app_util_platform.h"
#include "bsp_btn_ble.h"
#include "nrf_pwr_mgmt.h"

#if defined (UART_PRESENT)
#include "nrf_uart.h"
#endif
#if defined (UARTE_PRESENT)
#include "nrf_uarte.h"
#endif

#include "nrf_log.h"
#include "nrf_log_ctrl.h"
#include "nrf_log_default_backends.h"

typedef enum {
  // A tag identifying the SoftDevice BLE configuration.
  isp1807_app_ble_conn_cfg_tag = 1,

  // UUID type for the Nordic UART Service (vendor specific).
  isp1807_nus_service_uuid_type = BLE_UUID_TYPE_VENDOR_BEGIN,

  // Application's BLE observer priority. You shouldn't need to modify this value.
  isp1807_app_ble_observer_prio = 3,

  // The advertising interval (in units of 0.625 ms. This value corresponds to 40 ms).
  isp1807_app_adv_interval = 64,

  // The advertising duration (180 seconds) in units of 10 milliseconds.
  isp1807_app_adv_duration = 18000,

  // Minimum acceptable connection interval (20 ms), Connection interval uses 1.25 ms units.
  isp1807_min_conn_interval = MSEC_TO_UNITS(20, UNIT_1_25_MS),
  // Maximum acceptable connection interval (75 ms), Connection interval uses 1.25 ms units.
  isp1807_max_conn_interval = MSEC_TO_UNITS(75, UNIT_1_25_MS),
  // Slave latency.
  isp1807_slave_latency = 0,
  // Connection supervisory timeout (4 seconds), Supervision Timeout uses 10 ms units.
  isp1807_conn_sup_timeout = MSEC_TO_UNITS(4000, UNIT_10_MS),
  // Time from initiating event (connect or start of notification) to first time sd_ble_gap_conn_param_update is called (5 seconds).
  isp1807_first_conn_params_update_delay = APP_TIMER_TICKS(5000),
  // Time between each call to sd_ble_gap_conn_param_update after the first call (30 seconds).
  isp1807_next_conn_params_update_delay = APP_TIMER_TICKS(30000),
  // Number of attempts before giving up the connection parameter negotiation.
  isp1807_max_conn_params_update_count = 3,

  // Value used as error code on stack dump, can be used to identify stack location on stack unwind.
  isp1807_dead_beef = 0xDEADBEEF,

  // UART TX buffer size.
  isp1807_uart_tx_buf_size = 256,
  // UART RX buffer size.
  isp1807_uart_rx_buf_size = 256
} isp1807_Parameters;

// Handle of the current connection.
static uint16_t m_conn_handle = BLE_CONN_HANDLE_INVALID;
// Maximum length of data (in bytes) that can be transmitted to the peer by the Nordic UART service module.
static uint16_t m_ble_nus_max_data_len = BLE_GATT_ATT_MTU_DEFAULT - 3;

// Name of device. Will be included in the advertising data.
static const char* isp1807_device_name = "Nordic_UART";

// TODO YOUR_JOB: Use UUIDs for service(s) used in your application.
// Universally unique service identifier.
static ble_uuid_t m_adv_uuids[1]          =                                          
{
  {BLE_UUID_NUS_SERVICE, isp1807_nus_service_uuid_type}
};

// MACRO Definitions
// BLE NUS service instance.
BLE_NUS_DEF(m_nus, NRF_SDH_BLE_TOTAL_LINK_COUNT);
// GATT module instance.
NRF_BLE_GATT_DEF(m_gatt);
// Context for the Queued Write module.
NRF_BLE_QWR_DEF(m_qwr);
// Advertising module instance.
BLE_ADVERTISING_DEF(m_advertising);

// Function Prototypes
void init_isp1807();
void assert_nrf_callback(uint16_t line_num, const uint8_t *p_file_name);
static void init_gap_params(void);
static void nrf_qwr_error_handler(uint32_t nrf_error);
static void nus_data_handler(ble_nus_evt_t *p_evt);
static void init_services(void);
static void on_conn_params_evt(ble_conn_params_evt_t *p_evt);
static void conn_params_error_handler(uint32_t nrf_error);
static void init_conn_params(void);
static void sleep_mode_enter(void);
static void on_adv_evt(ble_adv_evt_t ble_adv_evt);
static void ble_evt_handler(ble_evt_t const *p_ble_evt, void *p_context);
static void init_ble_stack(void);
void init_gatt(void);
void bsp_event_handler(bsp_event_t event);
void uart_event_handle(app_uart_evt_t *p_event);
static void init_uart(void);
static void init_advertising(void);
void advertising_start(void);

#endif